%{

#include <string.h>	/* for strcmp, strdup & friends */
#include <stdio.h>	/* for strcmp, strdup & friends */
#include <stdlib.h>	/* for atoi() */

#include "parser.h"	/* (TK_) symbols from parser.y */

int line_number;	

%}

%x comment
%x asm

%%
[ \t\r]			break;		/* ignore white space */
"\n" 			++line_number; 

"("                     return TK_LPAR;
")"                     return TK_RPAR;
","                     return TK_COMMA;
";"                     return TK_SEMI;
"{"                     return TK_LACC;
"}"                     return TK_RACC; 

"true"                  return TK_TRUE;
"false"                 return TK_FALSE;
">"                     return TK_GREATER;
">="                    return TK_GREATER_EQ;
"<"                     return TK_LOWER;
"<="                    return TK_LOWER_EQ;
"=="                    return TK_EQUAL;
"!="                    return TK_NEQUAL;

"+"                     return TK_PLUS;
"-"                     return TK_MINUS;
"*"                     return TK_MUL;
"/"                     return TK_DIV;
"%"                     return TK_MOD;
"++"                    return TK_PLUSPLUS;

"["                     return TK_LBRAC;
"]"                     return TK_RBRAC;
"&"                     return TK_REF;
"."                     return TK_DOT;

"int"                   return TK_INT_TYPE;
"char"                  return TK_CHAR_TYPE;
"void"                  return TK_VOID_TYPE;
"struct"                return TK_STRUCT;
"typedef"               return TK_TYPEDEF;


"="                     return TK_ASSIGN; 

"if"                    return TK_IF;
"else"                  return TK_ELSE;
"while"                 return TK_WHILE;
"for"                   return TK_FOR;
"return"                return TK_RETURN;

"//"                    BEGIN(comment);   //
<comment>[^\n]          break;            //
<comment>\n             {                 // Comments handling
                          BEGIN(INITIAL); //
                          line_number++;  //
                        }                 //

[0-9]+                  { 
                          printf("(%s): ",yytext); 
                          return TK_NUM; 
                        }

\"[^\n]*\"              {
                          return TK_STRING; 
                        }


\'[^\n]\'               {
                          return TK_CHAR; 
                        }

[A-Za-z_]([A-Za-z0-9_])*"_t" {
                          printf("(%s): ",yytext); 
                          return TK_TYPE_ID;
                        }

[A-Za-z_]([A-Za-z0-9_])* {
                          printf("(%s): ",yytext); 
                          return TK_ID;
                        }

.			{ 
                          fprintf(stderr,"line %d: error: illegal character\n", 
                                  line_number); 
                          exit(1);
                        }


%%

int main(int argc, char* argv[])
{
  yyin = fopen(argv[1],"r");

  line_number = 1;
  while(1) 
    {
      int token = yylex();
      if(token == 0) break;

      printf("%d\n",token);
    }

  return 0;
}
